require "config"
require "locale";
require "closeButton";
require "scrollButton";


----------------------------------------------
-- Set initial values
----------------------------------------------
local locLabelBoxWidth, locLabelBoxHeight = 100, 20;
local featureSizeBoxWidth, featureSizeBoxHeight = 146, 43;

-- Set the height of the Vidiba window.
local addon_list_nlines = addon_list_nlines or 4 -- value from config.lua
local nVidiba = table.getn(Vidiba);
local nVidibaLinesMax = math.max(4, math.min(addon_list_nlines, nVidiba));
local VidibaFrameWidth, VidibaFrameHeight = 250, 110 + (20 * nVidibaLinesMax);
local VidibaScrollbarFrameW, VidibaScrollbarFrameH = 9, VidibaFrameHeight  - 110 - 20;

local nVidibaLines = nVidiba;
local VidibaScroll = 0;
local VidibaScrollCoeff = 6;
local VidibaScrollbarBoxH = 0;

local minFeatureSize = celestia:getminfeaturesize();

local featureSizeSliderFrameW = 130;
local featureSizeSliderBoxW = 12;
local featureSizeSliderMargin = 3;
local featureSizePosMax = featureSizeSliderFrameW - featureSizeSliderBoxW - (2 * featureSizeSliderMargin);
local featureSizePos = math.ceil(minFeatureSize / 100 * featureSizePosMax);

local addLeftOffset = 20;

local addBox = {};
local addCheckBox = {};
local addVisible = {};
local addLabelColor = {};

-- Disable all Location types.
local obs = celestia:getobserver();
local locFlags = obs:getlocationflags();
for k, v in pairs(locFlags) do
    locFlags[k] = false;
    obs:setlocationflags(locFlags);
end


local firstTickDone = false
local closeButtonDone = false

-- Enable Location labels.
celestia:setlabelflags{locations = true}

----------------------------------------------
-- Define local functions
----------------------------------------------

local setCheckBoxText = function(box, flag)
    if flag then
        box.Text = "x";
    else
        box.Text = "";
    end
end

local setAddCheckBoxText = function(box, k, flag)
    if flag then
        box[k].Text = "x";
    else
        box[k].Text = "";
    end
end


----------------------------------------------
-- Set up and Draw the boxes
----------------------------------------------
VidibaBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor({1, 1, 0, 1})
    --:fillcolor({1,1,0,.2})
    :movable(false)
    :attach(screenBox, width, height, 0, 0);

VidibaFrame = CXBox:new()
    :init(0, 0, 0, 0)
    :fillcolor(csetbofill)
    --:bordercolor(cbubordoff)
    :movable(false)
    :visible(false)
    :attach(screenBox, width/2+1, height-VidibaFrameHeight-2, width/2-VidibaFrameWidth-3, 2)
	
setVidibaButton = CXBox:new()
    :init(0, 0, 0, 0)
    :bordercolor(cbubordoff)
    --:fillcolor({1,1,0,.2})
    :textfont(normalfont)
    :textcolor(cbutextoff)
    :textpos("center")
    :movetext(0, 6)
    :text(_("Vi Di Ba"))
    :movable(false)
    :active(true)
    :attach(VidibaBox, 8, 4, 8, 3)


setVidibaButton.Action = function()
	VidibaFrame.Visible = not(VidibaFrame.Visible);
	if VidibaFrame.Visible then
		VidibaFrame:orderfront();
		setVidibaButton.Bordercolor = cbubordon;
		setVidibaButton.Fillcolor = {cbubordon[1]*2/3, cbubordon[2]*2/3, cbubordon[3]*2/3, cbubordon[4]/2};
	else
		setVidibaButton.Bordercolor = cbubordoff;
		setVidibaButton.Fillcolor = nil;
	end
end

	
VidibaFrame.Customdraw = function(this)
		textlayout:setfont(normalfont);
		textlayout:setfontcolor(ctext);
		textlayout:setpos(this.lb+addLeftOffset, this.tb-20);
		textlayout:println(_("VidiBa asteroids:"));

	if firstTickDone then
		if not(closeButtonDone) then
			closeButtonDone = true;
			makeCloseButton(VidibaCloseButton, VidibaFrame, setVidibaButton);
		end
	else
		firstTickDone = true;
	end
	VidibaFrame:attach(screenBox, width/2+1, height-VidibaFrameHeight-2, width/2-VidibaFrameWidth-3, 2)
end;


	
local makeAddBox = function(k, v, leftOffset, color)
    addBox[k] = CXBox:new()
        :init(0, 0, 0, 0)
        --:bordercolor(cbubordoff)
        --:fillcolor({1,1,0,.2})
        :textfont(normalfont)
        :textcolor(color)
        :movetext(0, 8)
        :text(_(v))
        :movable(false)
        :attach(VidibaFrame, leftOffset+15, VidibaFrameHeight-25-(20*k), VidibaFrameWidth-leftOffset-100, 14+(20*k))
end

local makeAddCheckBox = function(k, leftOffset)
    addCheckBox[k] = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(cbubordoff)
        --:fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(VidibaFrame, leftOffset, VidibaFrameHeight-25-(20*k), VidibaFrameWidth-leftOffset-11, 14+(20*k))
end




for k, v in ipairs(Vidiba) do

    if pcall( function() require (v) end ) then
        require(v)

        addLabelColor[k] = cbutextoff;
        if _G[v].labelcolor then
            addLabelColor[k] = _G[v].labelcolor;
        end

        makeAddBox(k, string.gsub(v, "_", " "), addLeftOffset, addLabelColor[k]);
        makeAddCheckBox(k, addLeftOffset);

        local vv, kk = v, k;

        if _G[v].objects then
            addVisible[k] = celestia:find(_G[v].objects[1]):visible();
        else
            addVisible[k] = false;
        end

        addBox[k].Visible = k <= nVidibaLinesMax;
        addCheckBox[k].Visible = k <= nVidibaLinesMax;

        addCheckBox[k].Customdraw =
            function(this)
                setAddCheckBoxText(addCheckBox, kk, addVisible[k + VidibaScroll]);
                addBox[k].Text = _(string.gsub(Vidiba[k + VidibaScroll], "_", " "));
                addBox[k].Textcolor = addLabelColor[k + VidibaScroll];
            end

            addCheckBox[k].Action = (function()
                return
                    function()
                        local vVidibaScroll = Vidiba[k + VidibaScroll];
                        addVisible[k + VidibaScroll] = not(addVisible[k + VidibaScroll]);
                        if _G[vVidibaScroll].objects then
                            for n, obj in pairs(_G[vVidibaScroll].objects) do
                                local addObj = celestia:find(obj);
                                addObj:setvisible(not addObj:visible());
                            end
                        end

                        if _G[vVidibaScroll].locationtypes then
                            for n, loctype in pairs(_G[vVidibaScroll].locationtypes) do
                                addLocType = string.lower(loctype);
                                local obs = celestia:getobserver();
                                local locFlags = obs:getlocationflags();
                                if locFlags[addLocType] ~= addVisible[k + VidibaScroll] then
                                    locFlags[addLocType] = addVisible[k + VidibaScroll];
                                    obs:setlocationflags(locFlags);
                                end
                            end
                        end

                        if _G[vVidibaScroll].script then
                            local scriptfile = "../Vidiba/"..vVidibaScroll.."/".._G[vVidibaScroll].script;
                            celestia:runscript(scriptfile);
                            if _G[vVidibaScroll].objects then
                                addVisible[k + VidibaScroll] = celestia:find(_G[vVidibaScroll].objects[1]):visible();
                                setAddCheckBoxText(addCheckBox, kk, addVisible[k + VidibaScroll]);
                            end
                        end
                    end
                end) ();

    end

end

VidibaScrollbarFrame = CXBox:new()
    --:bordercolor(cbubordoff)
    --:fillcolor({1, 0, 0, 1})
    :visible(nVidibaLines > nVidibaLinesMax)
    :attach(VidibaFrame, VidibaFrameWidth-VidibaScrollbarFrameW-3, VidibaFrameHeight-40-VidibaScrollbarFrameH, 3, 40);

VidibaScrollbarFrame.Customdraw =
    function(this)
        -- We add the scrollButton on the second tick
        -- when bounds are getting updated.
        if VidibaScroll_firstTickDone then
            if not(VidibaScrollButtonDone) then
                VidibaScrollButtonDone = true;
                makeScrollButton(VidibaScrollUpButton, VidibaScrollDownButton, VidibaScrollbarFrame);
            end
        else
            VidibaScroll_firstTickDone = true;
        end

        local lb3,bb3,rb3,tb3 =VidibaScrollbarFrame:bounds();
        local lb4,bb4,rb4,tb4 = VidibaScrollbarBox:bounds();
        VidibaScroll = math_round((tb3-tb4-3)/VidibaScrollCoeff);
        VidibaScrollbarFrame:attach(VidibaFrame, VidibaFrameWidth-VidibaScrollbarFrameW-3, VidibaFrameHeight-40-VidibaScrollbarFrameH, 3, 40);
    end

VidibaScrollbarBox = CXBox:new()
    :bordercolor(cbubordoff)
    :fillcolor(cscrollfill)
    :visible(true)
    :movable(true)
    :attach(VidibaScrollbarFrame, 0, (nVidibaLines-nVidibaLinesMax)*VidibaScrollCoeff, 0, 3)

featureSizeBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor({1, 1, 0, 1})
    --:fillcolor({1,1,0,.2})
    :movable(false)
    :attach(VidibaFrame, addLeftOffset-7, 25, 
                VidibaFrameWidth-featureSizeBoxWidth-addLeftOffset+7, VidibaFrameHeight-featureSizeBoxHeight-25);

featureSizeBox.Customdraw =
    function(this)
        local minFeatureSize = celestia:getminfeaturesize();
        textlayout:setpos(this.lb+7, this.tb-15);
        textlayout:setfont(normalfont);
        textlayout:setfontcolor(ctext);
        minFeatureSize_str = string.format("%1.0f", math.floor(minFeatureSize));
        textlayout:println(_("Minimum Feature Size:")..' '..minFeatureSize_str);
    end;

featureSizeSliderFrame = CXBox:new()
    --:bordercolor(cslidefrbord)
    --:fillcolor({.2,.2,.6,.1})
    :attach(featureSizeBox, 8, 8, 8, 24)
    :visible(true)

featureSizeSliderFrame.Customdraw =
    function(this)
        featureSize_lb = featureSizeSliderFrame.lb;
        featureSize_lb2 = featureSizeSliderBox.lb;
        featureSizeValue = 100 * math_round(featureSize_lb2 - featureSize_lb - 3) / featureSizePosMax;
        if featureSizeValue < 1 then
            featureSizeValue = 1;
        end
        if lastFeatureSizeValue ~= featureSizeValue and not(featureSizeFirstTick) then
            -- The slider has been moved; set the new Minimum Feature Size value.
            lastFeatureSizeValue = featureSizeValue;
            celestia:setminfeaturesize(featureSizeValue);
        end
        featureSizeFirstTick = false;
    end;

featureSizeSliderFrame2 = CXBox:new()
    :bordercolor(cslidefrbord)
    --:fillcolor({.2,.2,.6,.1})
    :attach(featureSizeSliderFrame, 2, 4, 2, 4)
    :visible(true)

featureSizeSliderBox = CXBox:new()
    :bordercolor(cslidebobord)
    :fillcolor(cslidebofill)
    :visible(true)
    :movable(true)
    :attach(featureSizeSliderFrame, featureSizeSliderMargin+featureSizePos, 0, 
                featureSizeSliderFrameW-featureSizeSliderMargin-featureSizePos-featureSizeSliderBoxW, 0);

featureSizeSliderFrame2.Masked = featureSizeSliderBox;

featureSizeSliderBox.Customdraw =
    function(this)
        local minFeatureSize = celestia:getminfeaturesize();
        if lastMinFeatureSize ~= minFeatureSize then
            -- Minimum Feature Size has been changed;
            -- set the new position of the slider.
            lastMinFeatureSize = minFeatureSize;
            featureSizePos = math_round(minFeatureSize / 100 * featureSizePosMax);
            featureSizeSliderBox:attach(featureSizeSliderFrame, featureSizeSliderMargin+featureSizePos, 0, 
                                                        featureSizeSliderFrameW-featureSizeSliderMargin-featureSizePos-featureSizeSliderBoxW, 0);
        end
    end;

locLabelBox = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor({1, 1, 0, 1})
    --:fillcolor({1,1,0,.2})
    :textfont(normalfont)
    :textcolor(ctext)
    :movetext(18, 3)
    :text(_("Label Features"))
    :movable(false)
    :attach(VidibaFrame, addLeftOffset-2, 10, 
                VidibaFrameWidth-locLabelBoxWidth-addLeftOffset+2, VidibaFrameHeight-locLabelBoxHeight-10);

locLabelCheckBox = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(cbubordoff)
        --:fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(0, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(locLabelBox, 2, 4, 87, 5)

setCheckBoxText(locLabelCheckBox, celestia:getlabelflags().locations)

locLabelBox.Customdraw =
    function(this)
        if locLabelFlag ~= celestia:getlabelflags().locations then
            locLabelFlag = celestia:getlabelflags().locations;
            setCheckBoxText(locLabelCheckBox, locLabelFlag)
        end
    end;

locLabelCheckBox.Action = (function()
        return
            function()
                local labelFlagTable = celestia:getlabelflags();
                labelFlagTable.locations = not(labelFlagTable.locations)
                celestia:setlabelflags(labelFlagTable);
            end
        end) ();
